/* @Project: 0507 Slingo Website redesign */
/* @File:  	 plugin.oop.js */
/* @Author:  Gregory Patmore*/
/* @Contact: gregorypatmore at gmail dot com*/
/* @Notes:   
	Handy tools for making object oriented javascript applications. 
	12/11 GP - Added:
		SLINGO.CallObj
		SLINGO.LinkedCall
		SLINGO.CallStack
		SLINGO.CallManager
		
	1/12 GP = Added : SLINGO.StateObj.usesStrictStates()
*/

/* Namespace verification overhead */
	/* check to see if Slingo is loaded and if the stars are aligned to load a plugin...  */
	if(typeof SLINGO == 'undefined') throw new Error('Cannot load plugin');
	/* Make sure the plugin doesn't exist already */
	if(SLINGO.OOP)throw new Error('SLINGO.OOP already defined');

SLINGO.NSCLASS = SLINGO.Class.extend({
	$_ : SLINGO 	
	,log : function(msg){return this.$_.triggerError(msg, this.$_.EWARN);}
	,warn : function(msg){if(typeof console != 'undefined') console.warn(msg);return this.log('WARNING: ' + msg);}
	,alert : function(msg){return this.$_.triggerError(msg, this.$_.EALERT);}
	,abort : function(msg){return this.$_.triggerError(msg, this.$_.ETRIGGER);}
	,dump : function(v){if(typeof console != 'undefined') console.dir(v);}
});
	
/************************************************************************
/************************************************************************
/* OOP Object Constructor
/************************************************************************
/* @PROTO:  RETURN SLINGO.OOP(PARAMS)
/* @DESC:   FUNCTION_DESCRIPTION
/* @RETURN: RETURN_VALUE_DESCRIPTION
/* @PARAM:  PARAM_VALUE_DESCRIPTION
/************************************************************************
/************************************************************************/
SLINGO.OOP = SLINGO.NSCLASS.extend({
	/* object class name */
	_obj_class_name : 'OOP',
	
	/* returns the object's class name property */
	getObjClassName : function(){return this._obj_class_name;}
});

/* Implementation of the Observable design pattern */
SLINGO.Observable = SLINGO.OOP.extend({
	/* Object class name as required by SLINGO.OOP */
	_obj_class_name : 'Observable',
	
	/* Object constructor as required by SLINGO.Class.extend EZ Class extender */
	init : function(){
		/* contains all the currently registered observers */
		this._obs={};
		/* specify a function in a child class to be called immediately after the object is initalized */
		if('_onInit' in this)this._onInit();
	},
	
	/* returns a boolean value indicating that there are registered observers for this event key */
	hasWatches : function(evtKey){
		e=evtKey.toUpperCase();
		return (e in this._obs && this._obs[e] instanceof Array && this._obs[e].length > 0);
	},
	
	/* returns a boolean value indicating whether this object has any registered observers for anything */
	isWatched : function(){
		for(var i in this._obs){
			if(this._obs[e].length > 0) return true;
		}
		return false;
	},
	
	/* gets all observers for a given event key, if none are found false is returned */
	getWatches : function(evtKey){
		e=evtKey.toUpperCase();
		/* send a copy */
		return (this.hasWatches(e)) ? this._obs[e].slice(0) : false;
	},
	
	/* registers a function to be notified on specified state changes */
	addWatch : function(eventKey, notify){
		e = eventKey.toUpperCase();
		if(typeof notify != 'function') throw 'Class:'+this.getObjClassName()+' method: addWatch: invalid function passed as argument for event key:'+eventKey;
		/* init the object in the observer list if not there already */
		if( !(e in this._obs) ) this._obs[e] = [];
		/* add the observer to the list */
		this._obs[e].push(notify);
		
		return true;
	},
	
	/* un-registers a function from being notified about a given state change */
	remWatch : function(eventKey,notify){
		e=eventKey.toUpperCase();
		if(e in this._obs){
			idx = this._obs[e].xfind(notify,true)
			if(idx !== false){
				this._obs[e].splice(idx,1);
			}else throw 'Class:'+this.getObjClassName()+' method:remWatch: notify function passed as argument not found for state:'+state;
		}else throw 'remWatch: no watchers found for state:'+state;
	},
	
	/* returns a boolean value indicating whether a given function (func) is a registered watcher for a given eventKey */
	isWatcher : function(eventKey, func){
		if(typeof func != 'function') 
			throw 'Class:'+this.getObjClassName()+' method: isWatcher: invalid function object passed as argument for event key:'+eventKey;
		
		e = eventKey.toUpperCase();
		
		if('ANY' in this._obs && SLINGO.isArray(this._obs['ANY']) && this._obs['ANY'].xfind(func, true) !== false)
			return true;
			
		if( !(e in this._obs) ) 
			return false; /* no watchers for that event key */
			
		return SLINGO.isArray(this._obs[e]) && this._obs[e].xfind(func,true) !== false;
	}
});

/* State Object - Manages the state of an object and allows register state change observers */
SLINGO.StateObj = SLINGO.Observable.extend({
	/* Object class name as required by SLINGO.OOP */
	_obj_class_name : 'StateObj',
		
	/* Object constructor as required by SLINGO.Class.extend EZ Class extender 
	 *
	 * @param bool strictStates [optional] whether to throw errors on state requests for undefined states.(defaults to true)
	 */
	init : function(strictStates){
		/* 12/11 GP added in for relaxed statenames ... more useful for cascading states to child references ... should work the same for old constructor calls _gp */
		this.strict_states = strictStates == null ? true : Boolean(strictStates);
		/* current state */
		this._curr_st='INIT';
		/* previous state */
		this._prev_st='NONE';
		/* current observers of state changes */
		this._obs={};
		/* function prefix to use for set state handlers */
		this.set_hdlr_pfx='_setState_';
		/* initialize the super class */
		this._super();
	},
	/* returns boolean value indicating whether this object uses strict states. */
	usesStrictStates : function(){
		return this.strict_states;
	},
	/* 'public' access to set the object to a given state */
	setState : function(state){
		
		//console.log('setState: ' + this._obj_class_name + ': ' + state);
	
		state=state.toUpperCase();
		hdlName = this.set_hdlr_pfx+state;
		
		/* if we have an available handler for this specific state. */
		if(hdlName in this){
			if(this[hdlName]() == false) /* failed the handler */
				return false;
		
		/* if there's no handler and we don't tolerate unknown states. */
		}else if(this.usesStrictStates){
			return SLINGO.triggerError(this.getObjClassName()+'::setState : unknown state received when using strict states.', SLINGO.ETRIGGER);
		}

		/* also need to run the ANY state */
		if(this._setState_ANY() == false){
			return false;
		}
		
		/* if we get here, call it a win. */
		if(!this._updState(state)){
			throw 'Class:'+this.getObjClassName()+' method:setState : object internal call to update state failed for state:'+state;
			
		}else return true;
	},
	/* returns the current state of the object */
	getState : function(){
		return ('onGetState' in this) ? this.onGetState() : this._curr_st.toUpperCase();
	},
	/* returns boolean value indicating whether the object contains a given state */
	hasState : function(state){
		return (this.set_hdlr_pfx+state.toUpperCase() in this);
	},
	/* returns boolean value indicating whether or not the object is currently in a given state */
	isState	 : function(state){
		return this.getState().toUpperCase() === state.toUpperCase();
	},
	/* returns the previous state of the object */
	getPrevState : function(){
		return this._prev_st.toUpperCase();
	},
	/* returns a boolean value indicating whether a given state is the previous state of the object */
	isPrevState : function(state){
		return state.toUpperCase() === this._prev_st.toUpperCase();
	},
	/* registers a function to be notified on specified state changes */
	addWatch : function(state, notify){
		if(state.toUpperCase() == 'ANY' || this.hasState(state) || !this.strict_states){
			try{
				return this._super(state,notify);
			}catch(e){
				if(typeof e == 'string'){
					throw this.getObjClassName()+'::addWatch : '+e;
				}else throw e;
			}
		}else throw 'Class:'+this.getObjClassName()+' method:addWatch: unknown state passed as argument, cannot add watcher for state:'+state;
	},
	/* un-registers a function from being notified about a given state change */
	remWatch : function(state,notify){
		us=state.toUpperCase();
		if(state.toUpperCase() == 'ANY' || this.hasState(us) || !this.strict_states){
			try{
				return this._super(us,notify);
			}catch(e){
				if(typeof e == 'string'){
					throw 'StateObj.'+e;
				}else throw e;
			}
		}else throw 'Class:'+this.getObjClassName()+' method:remWatch: unknown state passed as argument, cannot remove watcher for state:'+state;
	},
	/* notifies any registered observers that the objects state has changed */
	_notify : function(cs,ps){
		cs=cs.toUpperCase();
		/* check watches for this specific state */
		watches = this.getWatches(cs);
		if(watches){
			for(var i=0;i<watches.length;++i){
				this._callWatchFN(watches[i],cs,ps);
			}
		}
		/* check for watchers for any state change. */
		anyWatches = this.getWatches('ANY');
		if(anyWatches){
			for(var i=0;i<anyWatches.length;++i){
				this._callWatchFN(anyWatches[i],cs,ps);
			}
		}
		
		/* call optional onNotify method if defined */
		if('onNotify' in this) this.onNotify(cs,ps);
		
		return true;
	},
	/* 1/12 GP,NV Calls a given watcher function. override in child to adjust the arguments sent to watchers. */
	_callWatchFN : function(watchFN, cs, ps){
		watchFN(this,cs,ps);
		return true;
	},
	/* internal method to update the objects state. intended as priv */
	_updState : function(state){
		this._prev_st = this._curr_st;
		this._curr_st = state.toUpperCase();
		this._notify(this._curr_st, this._prev_st);
		/* optional _onUpdState method */
		if('_onUpdState' in this)  this._onUpdState(this._curr_st);
			
		return true;
	},
	/* state objects should support the 'ANY' state, for objects to be notified of any state changes. */
	_setState_ANY: function(){return true;}
});

/* a state object for a given DOM element on the page */
SLINGO.DisplayElement = SLINGO.StateObj.extend({
	/* Object class name as required by SLINGO.OOP */
	_obj_class_name : 'DisplayElement',
	
	/* Object constructor as required by SLINGO.Class.extend EZ Class extender */
	init:function(e, strictStates){
		if(!SLINGO.$(e)){
			throw 'Class:'+this.getObjClassName()+' method:DisplayElement: unknown element passed as argument';
		}else 
			e=SLINGO.$(e);
		//if(!e)throw 'Class:'+this.getObjClassName()+' method:DisplayElement: unknown element passed as argument';
		this._elm_ref=e;
		this._super(strictStates);
		e.DISPELM_REF = this;
	},
	
	/* return the html element this instance is attached to */
	getElement : function(){
		return this._elm_ref;
	},
	
	/* If it exists, it returns the DisplayElement instance attached to a given node. */
	getDisplayElementInstance : function(elm){
		elm = SLINGO.$(elm);
		if(!elm) throw 'Class:'+this.getObjClassName()+' method:DisplayElement.getDisplayElementInstance : invalid node passed as argument';
		return ('DISPELM_REF' in elm) ? elm.DISP_ELM_REF : false;
	},
	
	getElementWidth : function(){
		return parseInt(this.getElement().style.width);
	},
	
	getElementHeight : function(){
		return parseInt(this.getElement().style.height);
	}
});

/* Display (element) that is a composition of a number of display elements */
SLINGO.CompositeDisplay = SLINGO.DisplayElement.extend({
	/* Object class name as required by SLINGO.OOP */
	_obj_class_name : 'CompositeDisplay',
	
	/* Object constructor as required by SLINGO.Class.extend EZ Class extender */
	init : function(mainContainer, displayElms, strictStates){
		this._kids = {};
		if(typeof displayElms != 'undefined'){
			if(typeof displayElms == 'object' && !(displayElms instanceof Array)){
				for(var i in displayElms){
					this.addChild(i,displayElms[i]);
				}
			}
		}
		this._super(mainContainer, strictStates);
	},
	
	/* retreive the container element */
	getContainer : function(){
		return this.getElement();
	},
	
	/* returns boolean regarding if an element with a given reference id is registered with this object */
	isChild : function(refID){
		return (refID in this._kids);
	},
	
	/* returns a child element with a given reference id */
	getChild : function(refID){
		return (this.isChild(refID))?this._kids[refID] : null;
	},
	
	/* adds or updates a given child element */
	addChild : function(refID,element,override /* default false */){
		if(typeof override != 'undefined' || (!override && this.isChild(refID)))throw this.getObjClassName()+".addElement : reference id ("+refID+") passed as argument already registered, use 'override' parameter to update element";
		if(!(element instanceof SLINGO.DisplayElement))throw 'Class:'+this.getObjClassName()+' method:addElement : element passed as argument not a valid DisplayElement instance';
		element.COMPDISP_REF = this;
		this._kids[refID] = element;
		return this._kids[refID] == element;
	},

	/* remove a given element */
	remChild : function(refID){
		if(this.isChild(refID) != true) throw 'Class:'+this.getObjClassName()+' method:Composite.remElement : unknown element passed as argument';
		return delete this._kids[refID];
	},
	
	/* returns the full set of child elements */
	getAllChildren : function(){
		return this._kids;
	},

	/* direct child state access */

	/* set the state of a given child element */
	setChildState : function(refID, stateTo){
		if(this.isChild(refID) != true) throw 'Class:'+this.getObjClassName()+' method:Composite.setElementState : unknown element reference id passed as argument: refID: '+refID+', state: '+stateTo;
		return this.getChild(refID).setState(stateTo); 
	},
	
	/* retrieves the current state of a child element with a given reference */
	getChildState : function(refID){
		if(this.isChild(refID) != true) throw 'Class:'+this.getObjClassName()+' method:Composite.getElementState : unknown element reference id passed as argument';
		return this.getChild(refID).getState();
	},
	
	/* returns a boolean indicating whether an element with a given reference id is currently set to a given state */
	isChildState	: function(refID, state){
		if(this.isChild(refID) != true) throw 'Class:'+this.getObjClassName()+' method:Composite.isElementState : unknown element reference id passed as argument';
		return this.getChild(refID).isState(state);
	}
});

/* Simple Settings or parameters object */
SLINGO.Params = SLINGO.OOP.extend({
	
	/* Object class name as required by SLINGO.OOP */
	_obj_class_name : 'Params',
	
	/* object constructor as applied to SLINGO.Class EX class extender. */
	init : function (params){
		this._parms = new SLINGO.DataRegistry();
		
		if(typeof params != 'undefined')
			this.setParamList(params);
	},
	
	/* sets parameters from the property list of a given params object. */
	setParamList : function(params, overwrite){
		if(typeof params != 'object'){
			throw this.getObjClassName()+'::setParamList : invalid params object passed as argument. Must be of type [object]';
		}
		
		var i;
		for(i in params){
			this.setParam(i, params[i], overwrite);
		}
		
		return true;
	},
	
	/* sets a given param to a given val, if the param does not already exist it will attempt to create one. */
	setParam :function(param, val, overwrite){
		/* if the key is already set. */
		if(this._parms.isKey(param)){
			overwrite = Boolean(overwrite);
			/* if we aren't allowed to overwrite and the param key is not null, bail */
			if(!overwrite && this._parms.getItemValue(key) != null){
				return false;
			}
			
			/* or else we can write the parm */
			return this._parms.setItem(param, val);
			
		/* Not an existing parameter yet */
		}else{
			return this._parms.addItem(param, val);
		}
	},
	
	/* sets a given param to a given setVal if the current value is set to a given ifVal */
	setParamIfVal : function(param, setVal, ifVal, strict){
		return (this.isParamVal(param, ifVal, strict)) ? this.setParam(param, setVal, true) : false;
	},
	
	/* Checks to see if a given param exists in the parameters list  */
	hasParam :function(param){
		return this._parms.isKey(param);
	},
	
	/* Checks to see if a given param is currently set to a given val. */
	isParamVal :function(param, val, strict){
		var v = this._parms.getItemValue(param, true);
		return strict ? v === val : v == val;
	},
	
	/* returns parameter value for a given param, or null if the parameter does not exist. */
	getParam :function(param){
		return this._parms.getItemValue(param, true);
	},
	
	/* returns the entire list of params as an object */
	getParamList :function(){
		return this._parms.getTotalRegistry();
	},

	/* returns the data type of the value of a given param, or null if param does not exist. */
	getParamType :function(param){
		return (this._parms.isKey(param)) ? typeof this._parms.getItemValue(param) : null;
	},
	
	/* returns the number of parameters that have been set */
	countParams :function(){
		return this._parms.getNumItems();
	}
});

/* Handles calling an object and Capturing the result from the call */
SLINGO.CallObj = SLINGO.StateObj.extend({
	/* Object class name as required by SLINGO.OOP */
	_obj_class_name : 'CallObj',
	
	/* call object reference */
	_ref 		: null,
	/* Handle to the result  */
	_result 	: null,
	/* function to call */
	_func		: null, 
	/* scope to bind the call to (if none given, object will use the window object as the scope) */
	_scope 		: null,
	/* an array of arguments to send to the call (if any) */
	_args 		: null,
	/* an anonymous function wrapper that will be used when this.execute is called */
	_call		: null,
	
	/* Object constructor as required by SLINGO.Class.extend EZ Class extender 
	 * 
	 * @param string ref String reference for this object
	 * @param function func Function object to call
	 * @param object scope Object scope to call the function in
	 * @param array args [optional] array of arguments to send to the function when calling.
	 * @param bool strictStates [optional] whether to throw errors on state requests for undefined states.(defaults to true)
	 */
	init : function(ref, func, scope, args, strictStates){
		/* initialize the super class */
		this._super(strictStates);
		
		this._ref = String(ref);
		
		/* set the scope */
		if(scope == null){
			this._scope = window;
		}else if(typeof scope == 'object'){
			this._scope = scope;
		}else throw this.getObjClassName()+'::init : invalid object passed as scope argument. must be [object] type or null(uses window for global function)';
		
		/* set the function/method to call */
		if(typeof func == 'function'){
			this._func = func;
		}else{
			throw this.getObjClassName()+'::init : invalid method name passed as argument. must be a method of scope argument.';
		}
		
		/* set the arguments to call with */
		if(SLINGO.isArray(args)){
			this._args = args;
			
		}else if(args != null){
			throw this.getObjClassName()+'::init : invalid args passed as argument. must be a an array of arguments to call with, or null if no arguments are needed.';
		}
		
		/* now set up the call function */
		if(SLINGO.isArray(this._args)){
			this._call = function(){
				this._func.apply(this._scope);
			};
			
		}else{
			this._call = function(){
				this._func.apply(this._scope, this._args);
			};
		}
		
		/* set current state to ready so we know when it hasn't been called. */
		this._updState('READY');
	},
	
	/* returns the call reference identifier */
	getRef : function(){
		return this._ref;
	},
	
	/* checks to see if the objects reference id is a given ref. */
	isRef : function(ref, strict){
		return (strict) ? this._ref === String(ref) : this._ref == String(ref);
	},
	
	/* returns the given result, or null if no result was obtained. */
	getResult : function(){
		return this._result;
	},
	
	/* returns a boolean value indicating whether the object has procured a result from being called or not */
	hasResult : function(){
		return this._result != null;
	},
	
	/* returns a boolean value indicating whether the object call is complete or not. */
	isComplete : function(){
		return this.isState('COMPLETE');
	},
	
	/* returns a boolean value indicating whether the object is ready to be called or not. */
	isReady : function(){
		return this.isState('READY');
	},
	
	/* returns an object with the object, method, and arguments that will/was called */
	getCallInfo : function(){
		return {
			'call_scope' 		: this._scope,
			'call_method' 		: this._func,
			'call_arguments' 	: this._args
		};
	},
	
	/* execute's the call */
	execute : function(){
		if(this.isState('READY')){
			this._updState('EXECUTING');
			this._result = this._call();
			this._updState('COMPLETE');
			return this.hasResult();
		}else throw this.getObjClassName()+"::execute : cannot execute. object not in ready state.";
	},
	
	/* resets the object to be called again. */
	reset : function(force){
		if(!force && !this.isState('COMPLETE'))
			throw this.getObjClassName()+"::reset : cannot reset. object has not been completed. use force argument to override.";
			
		this._updState('RESETTING');
		this._result = null;
		this._updState('RESET');
		this._updState('READY');
		
		return this.isState('READY');
	},

	/* set state functions */
	_setState_READY 		: function(){return true;},
	_setState_EXECUTING 	: function(){return true;},
	_setState_COMPLETE 		: function(){return true;},
	_setState_RESETTING		: function(){return true;},
	_setState_RESET			: function(){return true;}
});

/* Handles a call to an object that is aware of being a part of a chain of call objects */
SLINGO.LinkedCall = SLINGO.CallObj.extend({
	/* Object class name as required by SLINGO.OOP */
	_obj_class_name : 'LinkedCall',
	
	/* counter to help in assigning references to child calls */
	_kid_ref_cntr : 0,
	
	/* handle to the parent call */
	_parent : null,
	
	/* DataRegistry object that handles references to child call objects. */
	_kids : null,
	
	/* Object constructor as required by SLINGO.Class.extend EZ Class extender 
	 * 
	 * @param string ref String reference for this object
	 * @param function func Function object to call
	 * @param object scope Object scope to call the function in
	 * @param array args [optional] array of arguments to send to the function when calling.
	 */
	init : function(ref, func, scope, args, strictStates){
		/* initialize the super class */
		this._super(ref, func, scope, args, strictStates);
	},
	
	/* sets the parent object for the current call. */
	setParent : function(linkedCall){
		if(linkedCall instanceof  SLINGO.LinkedCall){
			this._parent = linkedCall;
			this.addWatch('ANY', this._parent.updateChild);
			
		}else throw this.getObjClassName() + '::setParent : invalid object passed as parent.  Must be instance of SLINGO..LinkedCall';
	},
	
	/* Returns the parent call object, or null if none was set. */
	getParent : function(){
		return this._parent;
	},
	
	/* returns the number of children this object has set. If ifState is given it will return the number of children that are currently in ifState */
	getNumChildren : function(ifState){
		if(typeof ifState == 'string'){
			kids = this.getAllChildren(ifState);
			return SLINGO.isArray(kids) ? kids.length : 0;
			
		}else return (this._kids instanceof SLINGO.DataRegistry) ? this._kids.getNumItems() : 0;
	},
	
	/* returns a boolean value indicating whether this object currently has child calls associated with it. */
	hasChildren : function(){
		return this.getNumChildren() > 0;
	},
	
	/* checks to see if a given ref is a reference id to one of our children (or childrens children if checkChildren is set to true */
	isChildRef : function(ref, checkChildren){
		if(this._kids instanceof SLINGO.DataRegistry){
			/* if it's one of our kids, we don't have to go any further */
			if(this._kids.isKey(ref)) return true;
			
			if(Boolean(checkChildren)){
				reg = this._kids.getTotalRegistry();
				for(i in reg){
					if(reg[i].isChildRef(ref, true)) return true;
				}
			}
			
			/* cant find it */
			return false;
		}else return false;
	},
	
	/* returns a child object with a given ref or false if no child is found with given ref. */
	getChild : function(ref){
		if(this._kids instanceof SLINGO.DataRegistry){
			return this._kids.getItem(ref);
		}else return false;
	},
	
	/* returns all child calls or false if no children were found. if ifState is given, it will only return children currently in ifState */
	getAllChildren : function(ifState){
		if(this._kids instanceof SLINGO.DataRegistry){
			kids = this._kids.getAllValues();
			
			if(typeof ifState == 'string'){
				instate = [];
				var i;
				for(i = 0; i < kids.length; ++i){
					if(kids[i].isState(ifState)) instate.push(kids[i]);
				}
				
				return instate.length > 0 ? instate : false;
				
			}else return kids;
			
		}else return false;
	},
	
	/* returns the current state of a child call */
	getChildState : function(ref){
		kid = this.getChild(ref);
		if(kid instanceof SLINGO.LinkedCall){
			return kid.getState();
			
		}else throw this.getObjClassName()+'::getChildState : child('+ref+') not found';
	},
	
	/* returns an object with child_ref => state pairs, or false if no children are found. */
	getAllChildStates : function(){
		if(!this.hasChildren()) return false;
		kids = this._kids.getTotalRegistry();
		var i;
		states = {};
		for(i in kids) {states[i] = kids[i].getState();}
		return states;
	},
	
	/* returns a boolean value indicating whether any child calls associated are completed */
	hasCompleteChildren : function(){
		kids = this.getAllChildren('COMPLETE');
		return SLINGO.isArray(kids) && kids.length > 0; 
	},
	
	/* returns a boolean value indicating whether the object has incomplete child calls associated with it. */
	hasIncompleteChildren : function(){
		kids = this.getAllChildStates();
		if(typeof kids == 'object'){
			var i;
			for(i in kids){
				if(kids[i] != 'COMPLETE') return true;/* has* function. bail on first one not complete */
			}
			
			return false; /* all must be complete */
			
		}else return false; /* no kids found, therefore no incomplete kids */
	},
	
	/* handles notifications from child calls that a state change has happened. Will call an optional _onChildUpdState function if defined. */
	updateChild : function(child, currState, prevState){
		/* so far, can't think of anything to do in the current object, 
			but I wanted to have this place holder
				..at least that's what my gut is saying.. _gp */
				
		if('_onChildUpdState' in this)
			this._onChildUpdState(child, currState, prevState);
		
		return true;
	},
	
	/* adds a child call and returns it's reference */
	addChild : function(linkedCall, ref){
		if(linkedCall instanceof  SLINGO.LinkedCall){
		
			ref = (typeof ref == 'undefined') ? linkedCall.getRef() : String(ref);
			
			if(!this.isChildRef(ref, false)){
			
				if( !(this._kids instanceof SLINGO.DataRegistry) ) { this._kids = new SLINGO.DataRegistry(); }
				
				linkedCall.setParent(this);
				this._kids.addItem(ref, linkedCall);
				
				return ref;
				
			}else throw this.getObjClassName() + '::addChild : invalid child reference.  child reference('+ref+') already exists';
			
		}else throw this.getObjClassName() + '::addChild : invalid object passed as argument.  Must be instance of SLINGO.LinkedCall';
	},
	
	/* executes the call for a child with a given ref (if not already complete) */ 
	executeChild : function(ref){
		child = this.getChild(ref);
		if(child instanceof SLINGO.LinkedCall){
			if(!child.isComplete()){
				return child.execute();
			}else return true;
		}else throw this.getObjClassName()+'::executeChild : child('+ref+') not found'; 
	},
	
	/* executes all child calls (if not already complete) */
	executeAllChildren : function(){
		kids = this.getAllChildren();
		if(SLINGO.isArray(kids)){
			var i;
			for(i = 0; i < kids.length; ++i){
				if(!kids[i].isComplete()) {kids[i].execute();}
			}
		}
		
		return true;
	},
	
	/* returns the child call's result, or null if no result is found. */
	getChildResult : function(ref){
		child = this.getChild(ref);
		if(child instanceof SLINGO.LinkedCall){
			return (child.hasResult()) ? child.getResult() : null;
		}else throw this.getObjClassName()+'::getChildResult : child('+ref+') not found'; 
	},
	
	/* returns an object of child_ref => result pairs. a child reporting null result does not have one yet. 
		If ifState is given it will only return results for children currently in ifState 
	*/
	getAllChildResults : function(ifState){
		kids = this.getAllChildren(ifState);
		if(SLINGO.isArray(kids)){
			results = {};
			var i;
			for(i = 0; i < kids.length; ++i) { results[ kids[i].getRef() ] = kids[i].getResult(); }
			
			return results;
		}else return false;
	},
	
	/* resets a child call */
	resetChild : function(ref, force){
		kid = this.getChild(ref);
		if(kid instanceof SLINGO.LinkedCall){
			return kid.reset(force);
			
		}else throw this.getObjClassName()+'::resetChild : child('+ref+') not found';
	},
	
	/* resets all child calls */
	resetAllChildren : function(force){
		kids = this.getAllChildren();
		if(SLINGO.isArray(kids)){
			var i; 
			for(i = 0; i < kids.length; ++i){kids[i].reset(force);}
		}
		
		return true;
	},
	
	/* removes a child call association. if ifState is given, then it will only drop the child if it's currently in ifState */
	dropChild : function(ref, ifState){
		if(this.isChildRef(ref, false)){
			if(typeof ifState == 'string'){
			
				kid = this.getChild(ref);
				
				if(kid instanceof SLINGO.LinkedCall){
					return (kid.isState(ifState)) ? this._kids.removeItem(ref) : false;
					
				} else throw this.getObjClassName()+'::dropChild : child('+ref+') not a valid LinkedCall instance.  child seems corrupt.';
				
			}else return this._kids.removeItem(ref);
			
		}else throw this.getObjClassName()+'::dropChild : child('+ref+') not found';
	},
	
	/* removes a child call associations. if ifState is given, then it will only drop the children currently in ifState */
	dropAllChildren : function(ifState){
		kids = this.getAllChildren();
		if(SLINGO.isArray(kids)){
			var i;
			if(typeof ifState == 'string'){
				for(i = 0; i < kids.length; ++i){
					if(kids[i].isState(ifState)) 
						this._kids.removeItem(kids[i].getRef());
				}
				
			}else {
				for(i = 0; i < kids.length; ++i){
					this._kids.removeItem(kids[i].getRef());
				}
			}
			
			return true;
			
		/* no kids to be had */
		}else return true;
	},
	
	/* adds a state change notification to a given func for a given state on a child with a given ref */
	watchChild : function(ref, state, func){
		kid = this.getChild(ref);
		if(kid instanceof SLINGO.LinkedCall){
			return kid.addWatch(state, func);
		}else throw this.getObjClassName()+'::watchChild : child('+ref+') not found';
	},
	
	/* removes state change notifications to a given func for a given state on a child with a given ref */
	unwatchChild : function(ref, state, func){
		kid = this.getChild(ref);
		if(kid instanceof SLINGO.LinkedCall){
			return kid.remWatch(state, func);
		}else throw this.getObjClassName()+'::unwatchChild : child('+ref+') not found';
	},
	
	/* Creates a new LinkedCall object, registers it as a child, and returns the reference */
	makeChild : function(func, scope, args, strictStates){
		strictStates = strictStates == null ? this.strict_states : Boolean(strictStates);
		c = new SLINGO.LinkedCall(this.getNextRef(), func, scope, args, strictStates);
		ref = this.addChild(c);
		return ref;
	},
	
	/* resets the call.  optional resetChildren param will also reset all the object's child calls */
	reset : function(force, resetChildren){
		ret = this._super(force);
		kret = (Boolean(resetChildren)) ? this.resetAllChildren() : true; 
		return ret && kret;
	},
	
	/* generates the next available ref id to assign to a child task */
	_getNextRef : function(){
		while(true){
			++this._kid_ref_cntr;
			ref = this._ref+'.'+this._kid_ref_cntr;
			if(!this.isChildRef(ref, false)) return ref;
		}
	}
});

/* OK, OOP plugin finished. */
SLINGO.transitLoadComplete = true;